@@ -33,12 +33,14 @@
 
 :- module(prolog_clause,
 	  [ clause_info/4,		% +ClauseRef, -File, -TermPos, -VarNames
+	    initialization_layout/4,	% +SourceLoc, +Goal, -Term, -TermPos
 	    predicate_name/2,		% +Head, -Name
 	    clause_name/2		% +ClauseRef, -Name
 	  ]).
 :- use_module(library(lists), [append/3]).
 :- use_module(library(occurs), [sub_term/2]).
 :- use_module(library(debug)).
+:- use_module(library(option)).
 :- use_module(library(listing)).
 :- use_module(library(prolog_source)).
 
@@ -61,15 +63,32 @@ dynamic predicates using listing to  XPCE   objects  to  this. Note that
 clause_info/4 as below can be slow.
 */
 
-%%	clause_info(+ClauseRef, -File, -TermPos, -VarNames)
+%%	clause_info(+ClauseRef, -File, -TermPos, -VarOffsets) is semidet.
+%%	clause_info(+ClauseRef, -File, -TermPos, -VarOffsets, +Options) is semidet.
 %
 %	Fetches source information for the  given   clause.  File is the
 %	file from which the clause  was   loaded.  TermPos describes the
 %	source layout in a format   compatible  to the subterm_positions
-%	option of read_term/2.  VarNames provides access to the variable
-%	allocation in a stack-frame.  See make_varnames/5 for details.
+%	option  of  read_term/2.  VarOffsets  provides   access  to  the
+%	variable allocation in a stack-frame.   See  make_varnames/5 for
+%	details.
+%
+%	Note that positions are  _|character   positions|_,  i.e., _not_
+%	bytes. Line endings count as a   single character, regardless of
+%	whether the actual ending is =|\n|= or =|\r\n|_.
+%
+%	Defined options are:
+%
+%	  * variable_names(-Names)
+%	  Unify Names with the variable names list (Name=Var) as
+%	  returned by read_term/3.  This argument is intended for
+%	  reporting source locations and refactoring based on
+%	  analysis of the compiled code.
 
 clause_info(ClauseRef, File, TermPos, NameOffset) :-
+	clause_info(ClauseRef, File, TermPos, NameOffset, []).
+
+clause_info(ClauseRef, File, TermPos, NameOffset, Options) :-
 	(   debugging(clause_info)
 	->  clause_name(ClauseRef, Name),
 	    debug(clause_info, 'clause_info(~w) (~w)... ',
@@ -90,6 +109,7 @@ clause_info(ClauseRef, File, TermPos, NameOffset) :-
 	),
 	debug(clause_info, 'from ~w:~d ... ', [File, LineNo]),
 	read_term_at_line(File, LineNo, Module, Clause, TermPos0, VarNames),
+	option(variable_names(VarNames), Options, _),
 	debug(clause_info, 'read ...', []),
 	unify_clause(Clause, DecompiledClause, Module, TermPos0, TermPos),
 	debug(clause_info, 'unified ...', []),
@@ -148,7 +168,14 @@ unify_args(I, Arity, T1, T2) :-
 %	Read a term from File at Line.
 
 read_term_at_line(File, Line, Module, Clause, TermPos, VarNames) :-
+	setup_call_cleanup(
+	    '$push_input_context'(clause_info),
+	    read_term_at_line_2(File, Line, Module, Clause, TermPos, VarNames),
+	    '$pop_input_context').
+
+read_term_at_line_2(File, Line, Module, Clause, TermPos, VarNames) :-
 	catch(open(File, read, In), _, fail),
+	set_stream(In, newline(detect)),
 	call_cleanup(
 	    read_source_term_at_location(
 		In, Clause,
@@ -226,13 +253,13 @@ find_varname(Var, [_|T], Name) :-
 
 unify_clause(Read, Read, _, TermPos, TermPos) :- !.
 					% XPCE send-methods
-unify_clause(Read, Decompiled, Module, TermPoso, TermPos) :-
-	unify_clause_hook(Read, Decompiled, Module, TermPoso, TermPos), !.
-unify_clause(:->(Head, Body), (PlHead :- PlBody), _, TermPos0, TermPos) :- !,
-	pce_method_clause(Head, Body, PlHead, PlBody, TermPos0, TermPos).
+unify_clause(Read, Decompiled, Module, TermPos0, TermPos) :-
+	unify_clause_hook(Read, Decompiled, Module, TermPos0, TermPos), !.
+unify_clause(:->(Head, Body), (PlHead :- PlBody), M, TermPos0, TermPos) :- !,
+	pce_method_clause(Head, Body, PlHead, PlBody, M, TermPos0, TermPos).
 					% XPCE get-methods
-unify_clause(:<-(Head, Body), (PlHead :- PlBody), _, TermPos0, TermPos) :- !,
-	pce_method_clause(Head, Body, PlHead, PlBody, TermPos0, TermPos).
+unify_clause(:<-(Head, Body), (PlHead :- PlBody), M, TermPos0, TermPos) :- !,
+	pce_method_clause(Head, Body, PlHead, PlBody, M, TermPos0, TermPos).
 					% Unit test clauses
 unify_clause((TH :- Body),
 	     (_:'unit body'(_, _) :- !, Body), _,
@@ -251,6 +278,7 @@ unify_clause((Head :- Read),
 				 [ PH,
 				   term_position(0,0,0,0,[0-0,PB])
 				 ]).
+					% DCG rules
 unify_clause(Read, Compiled1, Module, TermPos0, TermPos) :-
 	Read = (_ --> List, _),
 	is_list(List),
@@ -264,11 +292,11 @@ unify_clause(Read, Compiled1, Module, TermPos0, TermPos) :-
 					     term_position(_,_,_,_,[_,BP])
 					   ]), !,
 	TermPos1 = term_position(F,T,FF,FT,[ HP, BP ]),
-	match_module(Compiled2, Compiled1, TermPos1, TermPos).
+	match_module(Compiled2, Compiled1, Module, TermPos1, TermPos).
 					% general term-expansion
 unify_clause(Read, Compiled1, Module, TermPos0, TermPos) :-
 	ci_expand(Read, Compiled2, Module),
-	match_module(Compiled2, Compiled1, TermPos0, TermPos).
+	match_module(Compiled2, Compiled1, Module, TermPos0, TermPos).
 					% I don't know ...
 unify_clause(_, _, _, _, _) :-
 	debug(clause_info, 'Could not unify clause', []),
@@ -279,16 +307,27 @@ unify_clause_head(H1, H2) :-
 	strip_module(H2, _, H).
 
 ci_expand(Read, Compiled, Module) :-
-	catch(setup_call_cleanup('$set_source_module'(Old, Module),
-				 expand_term(Read, Compiled),
-				 '$set_source_module'(_, Old)),
+	catch(setup_call_cleanup(
+		  ( set_xref_flag(OldXRef),
+		    '$set_source_module'(Old, Module)
+		  ),
+		  expand_term(Read, Compiled),
+		  ( '$set_source_module'(_, Old),
+		    set_prolog_flag(xref, OldXRef)
+		  )),
 	      E,
 	      expand_failed(E, Read)).
 
-match_module((H1 :- B1), (H2 :- B2), Pos0, Pos) :- !,
+set_xref_flag(Value) :-
+	current_prolog_flag(xref, Value), !,
+	set_prolog_flag(xref, true).
+set_xref_flag(false) :-
+	create_prolog_flag(xref, true, [type(boolean)]).
+
+match_module((H1 :- B1), (H2 :- B2), Module, Pos0, Pos) :- !,
 	unify_clause_head(H1, H2),
-	unify_body(B1, B2, Pos0, Pos).
-match_module(H1, H2, Pos, Pos) :-	% deal with facts
+	unify_body(B1, B2, Module, Pos0, Pos).
+match_module(H1, H2, _, Pos, Pos) :-	% deal with facts
 	unify_clause_head(H1, H2).
 
 %%	expand_failed(+Exception, +Term)
@@ -301,19 +340,19 @@ expand_failed(E, Read) :-
 	debug(clause_info, 'Term-expand ~p failed: ~w', [Read, Msg]),
 	fail.
 
-%%	unify_body(+Read, +Decompiled, +Pos0, -Pos)
+%%	unify_body(+Read, +Decompiled, +Module, +Pos0, -Pos)
 %
 %	Deal with translations implied by the compiler.  For example,
 %	compiling (a,b),c yields the same code as compiling a,b,c.
 %
 %	Pos0 and Pos still include the term-position of the head.
 
-unify_body(B, B, Pos, Pos) :-
+unify_body(B, B, _, Pos, Pos) :-
 	does_not_dcg_after_binding(B, Pos), !.
-unify_body(R, D,
+unify_body(R, D, Module,
 	   term_position(F,T,FF,FT,[HP,BP0]),
 	   term_position(F,T,FF,FT,[HP,BP])) :-
-	ubody(R, D, BP0, BP).
+	ubody(R, D, Module, BP0, BP).
 
 %%	does_not_dcg_after_binding(+ReadBody, +ReadPos) is semidet.
 %
@@ -337,69 +376,83 @@ a --> { x, y, z }.
     which the compiler creates "a(X,Y) :- x, y, z, X=Y".
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-%%	ubody(+Read, +Decompiled, +TermPosRead, -TermPosForDecompiled)
+%%	ubody(+Read, +Decompiled, +Module, +TermPosRead, -TermPosForDecompiled)
 %
 %	@param Read		Clause read _after_ expand_term/2
 %	@param Decompiled	Decompiled clause
+%	@param Module		Load module
 %	@param TermPosRead	Sub-term positions of source
 
-ubody(B, B, P, P) :-
+ubody(B, B, _, P, P) :-
 	does_not_dcg_after_binding(B, P), !.
-ubody(X, call(X),			% X = call(X)
-      From-To,
-      term_position(From, To, From, To, [From-To])) :- !.
-ubody(B0, B,
+ubody(M:R, D, M, term_position(_,_,_,_,[_,RP]), TPOut) :-
+	ubody(R, D, M, RP, TPOut).
+ubody(X, call(X), _,			% X = call(X)
+      Pos,
+      term_position(From, To, From, To, [Pos])) :- !,
+	arg(1, Pos, From),
+	arg(2, Pos, To).
+ubody(B0, B, M,
       brace_term_position(F,T,A0),
       Pos) :-
 	B0 = (_,_=_), !,
 	T1 is T - 1,
-	ubody(B0, B,
+	ubody(B0, B, M,
 	      term_position(F,T,
 			    F,T,
 			    [A0,T1-T]),
 	      Pos).
-ubody(B0, B,
+ubody(B0, B, M,
       brace_term_position(F,T,A0),
       term_position(F,T,F,T,[A])) :- !,
-	ubody(B0, B, A0, A).
-ubody(C0, C, P0, P) :-
+	ubody(B0, B, M, A0, A).
+ubody(C0, C, M, P0, P) :-
 	nonvar(C0), nonvar(C),
 	C0 = (_,_), C = (_,_), !,
 	conj(C0, P0, GL, PL),
-	mkconj(C, P, GL, PL).
-ubody(X0, X,
+	mkconj(C, M, P, GL, PL).
+ubody(X0, X, M,
       term_position(F,T,FF,TT,PA0),
       term_position(F,T,FF,TT,PA)) :-
-	meta(X0), !,
+	meta(M, X0, S), !,
 	X0 =.. [_|A0],
 	X  =.. [_|A],
-	ubody_list(A0, A, PA0, PA).
+	S =.. [_|AS],
+	ubody_list(A0, A, AS, M, PA0, PA).
+ubody(X0, X, M,
+      term_position(F,T,FF,TT,PA0),
+      term_position(F,T,FF,TT,PA)) :-
+	expand_goal(X0, X, M, PA0, PA).
+
 					% 5.7.X optimizations
-ubody(_=_, true,			% singleton = Any
+ubody(_=_, true, _,			% singleton = Any
       term_position(F,T,_FF,_TT,_PA),
       F-T) :- !.
-ubody(_==_, fail,			% singleton/firstvar == Any
+ubody(_==_, fail, _,			% singleton/firstvar == Any
       term_position(F,T,_FF,_TT,_PA),
       F-T) :- !.
-ubody(A1=B1, B2=A2,			% Term = Var --> Var = Term
+ubody(A1=B1, B2=A2, _,			% Term = Var --> Var = Term
       term_position(F,T,FF,TT,[PA1,PA2]),
       term_position(F,T,FF,TT,[PA2,PA1])) :-
 	(A1==B1) =@= (B2==A2), !,
 	A1 = A2, B1=B2.
-ubody(A1==B1, B2==A2,			% const == Var --> Var == const
+ubody(A1==B1, B2==A2, _,		% const == Var --> Var == const
       term_position(F,T,FF,TT,[PA1,PA2]),
       term_position(F,T,FF,TT,[PA2,PA1])) :-
 	(A1==B1) =@= (B2==A2), !,
 	A1 = A2, B1=B2.
-ubody(A is B - C, A is B + C2, Pos, Pos) :-
+ubody(A is B - C, A is B + C2, _, Pos, Pos) :-
 	integer(C),
 	C2 =:= -C, !.
 
-ubody_list([], [], [], []).
-ubody_list([G0|T0], [G|T], [PA0|PAT0], [PA|PAT]) :-
-	ubody(G0, G, PA0, PA),
-	ubody_list(T0, T, PAT0, PAT).
+ubody_list([], [], [], _, [], []).
+ubody_list([G0|T0], [G|T], [AS|ASL], M, [PA0|PAT0], [PA|PAT]) :-
+	ubody_elem(AS, G0, G, M, PA0, PA),
+	ubody_list(T0, T, ASL, M, PAT0, PAT).
 
+ubody_elem(0, G0, G, M, PA0, PA) :- !,
+	ubody(G0, G, M, PA0, PA).
+ubody_elem(_, G, G, _, PA, PA).
 
 conj(Goal, Pos, GoalList, PosList) :-
 	conj(Goal, Pos, GoalList, [], PosList, []).
@@ -418,16 +471,16 @@ conj((!,(S=SR)), F-T, [!,S=SR|TG], TG, [F-T,F1-T1|TP], TP) :-
 conj(A, P, [A|TG], TG, [P|TP], TP).
 
 
-mkconj(Goal, Pos, GoalList, PosList) :-
-	mkconj(Goal, Pos, GoalList, [], PosList, []).
+mkconj(Goal, M, Pos, GoalList, PosList) :-
+	mkconj(Goal, M, Pos, GoalList, [], PosList, []).
 
-mkconj(Conj, term_position(0,0,0,0,[PA,PB]), GL, TG, PL, TP) :-
+mkconj(Conj, M, term_position(0,0,0,0,[PA,PB]), GL, TG, PL, TP) :-
 	nonvar(Conj),
 	Conj = (A,B), !,
-	mkconj(A, PA, GL, TGA, PL, TPA),
-	mkconj(B, PB, TGA, TG, TPA, TP).
-mkconj(A0, P0, [A|TG], TG, [P|TP], TP) :-
-	ubody(A, A0, P, P0).
+	mkconj(A, M, PA, GL, TGA, PL, TPA),
+	mkconj(B, M, PB, TGA, TG, TPA, TP).
+mkconj(A0, M, P0, [A|TG], TG, [P|TP], TP) :-
+	ubody(A, A0, M, P, P0).
 
 
 		 /*******************************
@@ -444,20 +497,20 @@ mapped to:
 
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-pce_method_clause(Head, Body, _:PlHead, PlBody, TermPos0, TermPos) :- !,
-	pce_method_clause(Head, Body, PlBody, PlHead, TermPos0, TermPos).
+pce_method_clause(Head, Body, M:PlHead, PlBody, _, TermPos0, TermPos) :- !,
+	pce_method_clause(Head, Body, PlBody, PlHead, M, TermPos0, TermPos).
 pce_method_clause(Head, Body,
 		  send_implementation(_Id, Msg, Receiver), PlBody,
-		  TermPos0, TermPos) :- !,
+		  M, TermPos0, TermPos) :- !,
 	debug(clause_info, 'send method ...', []),
 	arg(1, Head, Receiver),
 	functor(Head, _, Arity),
 	pce_method_head_arguments(2, Arity, Head, Msg),
 	debug(clause_info, 'head ...', []),
-	pce_method_body(Body, PlBody, TermPos0, TermPos).
+	pce_method_body(Body, PlBody, M, TermPos0, TermPos).
 pce_method_clause(Head, Body,
 		  get_implementation(_Id, Msg, Receiver, Result), PlBody,
-		  TermPos0, TermPos) :- !,
+		  M, TermPos0, TermPos) :- !,
 	debug(clause_info, 'get method ...', []),
 	arg(1, Head, Receiver),
 	debug(clause_info, 'receiver ...', []),
@@ -468,7 +521,7 @@ pce_method_clause(Head, Body,
 	Ar is Arity - 1,
 	pce_method_head_arguments(2, Ar, Head, Msg),
 	debug(clause_info, 'head ...', []),
-	pce_method_body(Body, PlBody, TermPos0, TermPos).
+	pce_method_body(Body, PlBody, M, TermPos0, TermPos).
 
 pce_method_head_arguments(N, Arity, Head, Msg) :-
 	N =< Arity, !,
@@ -487,7 +540,7 @@ pce_unify_head_arg(V, A) :-
 pce_unify_head_arg(A:_=_, A) :- !.
 pce_unify_head_arg(A:_, A).
 
-%	pce_method_body(+SrcBody, +DbBody, +TermPos0, -TermPos
+%	pce_method_body(+SrcBody, +DbBody, +M, +TermPos0, -TermPos
 %
 %	Unify the body of an XPCE method.  Goal-expansion makes this
 %       rather tricky, especially as we cannot call XPCE's expansion
@@ -500,7 +553,7 @@ pce_unify_head_arg(A:_, A).
 %	starts with an I_CONTEXT call. This implies we need a
 %	hypothetical term-position for the module-qualifier.
 
-pce_method_body(A0, A, TermPos0, TermPos) :-
+pce_method_body(A0, A, M, TermPos0, TermPos) :-
 	TermPos0 = term_position(F, T, FF, FT,
 				 [ HeadPos,
 				   BodyPos0
@@ -509,14 +562,14 @@ pce_method_body(A0, A, TermPos0, TermPos) :-
 				 [ HeadPos,
 				   term_position(0,0,0,0, [0-0,BodyPos])
 				 ]),
-	pce_method_body2(A0, A, BodyPos0, BodyPos).
+	pce_method_body2(A0, A, M, BodyPos0, BodyPos).
 
 
-pce_method_body2(::(_,A0), A, TermPos0, TermPos) :- !,
+pce_method_body2(::(_,A0), A, M, TermPos0, TermPos) :- !,
 	TermPos0 = term_position(_, _, _, _, [_Cmt,BodyPos0]),
 	TermPos  = BodyPos,
-	expand_goal(A0, A, BodyPos0, BodyPos).
-pce_method_body2(A0, A, TermPos0, TermPos) :-
+	expand_goal(A0, A, M, BodyPos0, BodyPos).
+pce_method_body2(A0, A, M, TermPos0, TermPos) :-
 	A0 =.. [Func,B0,C0],
 	control_op(Func), !,
 	A =.. [Func,B,C],
@@ -528,10 +581,10 @@ pce_method_body2(A0, A, TermPos0, TermPos) :-
 				 [ BP,
 				   CP
 				 ]),
-	pce_method_body2(B0, B, BP0, BP),
-	expand_goal(C0, C, CP0, CP).
-pce_method_body2(A0, A, TermPos0, TermPos) :-
-	expand_goal(A0, A, TermPos0, TermPos).
+	pce_method_body2(B0, B, M, BP0, BP),
+	expand_goal(C0, C, M, CP0, CP).
+pce_method_body2(A0, A, M, TermPos0, TermPos) :-
+	expand_goal(A0, A, M, TermPos0, TermPos).
 
 control_op(',').
 control_op((;)).
@@ -555,40 +608,36 @@ To keep track of the source-locations, we   have to redo the analysis of
 the clause as defined in init.pl
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-expand_goal(G, call(G), P, term_position(0,0,0,0,[P])) :-
+expand_goal(G, call(G), _, P, term_position(0,0,0,0,[P])) :-
         var(G), !.
-expand_goal(G, G, P, P) :-
+expand_goal(G, G, _, P, P) :-
         var(G), !.
-expand_goal(M0, M, P0, P) :-
-	meta(M0), !,
+expand_goal(M0, M, Module, P0, P) :-
+	meta(Module, M0, S), !,
 	P0 = term_position(F,T,FF,FT,PL0),
 	P  = term_position(F,T,FF,FT,PL),
 	functor(M0, Functor, Arity),
 	functor(M,  Functor, Arity),
-	expand_meta_args(PL0, PL, 1, M0, M).
-expand_goal(A, B, P0, P) :-
+	expand_meta_args(PL0, PL, 1, S, Module, M0, M).
+expand_goal(A, B, Module, P0, P) :-
         goal_expansion(A, B0, P0, P1), !,
-	expand_goal(B0, B, P1, P).
-expand_goal(A, A, P, P).
+	expand_goal(B0, B, Module, P1, P).
+expand_goal(A, A, _, P, P).
 
-expand_meta_args([], [], _, _, _).
-expand_meta_args([P0|T0], [P|T], I, M0, M) :-
+expand_meta_args([],      [],   _,  _, _,      _,  _).
+expand_meta_args([P0|T0], [P|T], I, S, Module, M0, M) :-
 	arg(I, M0, A0),
 	arg(I, M,  A),
-	expand_goal(A0, A, P0, P),
+	arg(I, S,  AS),
+	expand_arg(AS, A0, A, Module, P0, P),
 	NI is I + 1,
-	expand_meta_args(T0, T, NI, M0, M).
+	expand_meta_args(T0, T, NI, S, Module, M0, M).
 
-meta((_  ,  _)).
-meta((_  ;  _)).
-meta((_  -> _)).
-meta((_ *-> _)).
-meta((\+ _)).
-meta((not(_))).
-meta((call(_))).
-meta((once(_))).
-meta((ignore(_))).
-meta((forall(_, _))).
+expand_arg(0, A0, A, Module, P0, P) :- !,
+	expand_goal(A0, A, Module, P0, P).
+expand_arg(_, A, A, _, P, P).
+
+meta(M, G, S) :- predicate_property(M:G, meta_predicate(S)).
 
 goal_expansion(send(R, Msg), send_class(R, _, SuperMsg), P, P) :-
 	compound(Msg),
@@ -625,6 +674,27 @@ goal_expansion(G0, G, P, P) :-
 	G0 \== G.			% \=@=?
 
 
+		 /*******************************
+		 *	  INITIALIZATION	*
+		 *******************************/
+
+%%	initialization_layout(+SourceLocation, ?InitGoal,
+%%			      -ReadGoal, -TermPos) is semidet.
+%
+%	Find term-layout of :- initialization directives.
+
+initialization_layout(File:Line, M:Goal0, Goal, TermPos) :-
+	read_term_at_line(File, Line, M, Directive, DirectivePos, _),
+	Directive    = (:- initialization(ReadGoal)),
+	DirectivePos = term_position(_, _, _, _, [InitPos]),
+	InitPos      = term_position(_, _, _, _, [GoalPos]),
+	(   ReadGoal = M:_
+	->  Goal = M:Goal0
+	;   Goal = Goal0
+	),
+	unify_body(ReadGoal, Goal, M, GoalPos, TermPos), !.
+
+
 		 /*******************************
 		 *	  PRINTABLE NAMES	*
 		 *******************************/
@@ -671,4 +741,8 @@ clause_name(Ref, Name) :-
 	predicate_name(Head, PredName),
 	thaffix(N, Th),
 	format(string(Name), '~d-~w clause of ~w', [N, Th, PredName]).
+clause_name(Ref, Name) :-
+	clause_property(Ref, erased), !,
+	clause_property(Ref, predicate(M:PI)),
+	format(string(Name), 'erased clause from ~q', [M:PI]).
 clause_name(_, '<meta-call>').
