@@ -82,8 +82,7 @@ users of the library are:
 
 
 :- predicate_options(prolog_read_source_term/4, 4,
-		     [ process_comment(boolean),
-		       pass_to(system:read_term/3, 3)
+		     [ pass_to(system:read_clause/3, 3)
 		     ]).
 :- predicate_options(read_source_term_at_location/3, 3,
 		     [ line(integer),
@@ -117,38 +116,10 @@ users of the library are:
 %	       arbitrary location.
 
 prolog_read_source_term(In, Term, Expanded, Options) :-
-	read_source_term(In, Term, Options),
+	read_clause(In, Term, Options),
 	expand(Term, In, Expanded),
 	update_state(Expanded).
 
-:- multifile
-	prolog:comment_hook/3.
-
-read_source_term(In, Term, Options) :-
-	'$get_predicate_attribute'(prolog:comment_hook(_,_,_),
-				   number_of_clauses, N),
-	N > 0,
-	option(process_comment(true), Options), !,
-	select_option(term_position(TermPos), Options, Options1, _),
-	'$set_source_module'(SM, SM),
-	read_term(In, Term,
-		  [ comments(Comments),
-		    module(SM),
-		    term_position(TermPos)
-		  | Options1
-		  ]),
-	(   catch(prolog:comment_hook(Comments, TermPos, Term), E,
-		  print_message(error, E))
-	->  true
-	;   true
-	).
-read_source_term(In, Term, Options) :-
-	'$set_source_module'(SM, SM),
-	read_term(In, Term,
-		  [ module(SM)
-		  | Options
-		  ]).
-
 :- public
 	expand/3.			% Used by Prolog colour
 
@@ -306,16 +277,18 @@ read_source_term_at_location(Stream, Term, Options) :-
 	debug(read, 'Using syntax ~w (c=~w)', [Syntax, X]),
 	push_operators(Module:Ops),
 	Setup,
-	catch(read_term(Stream, Term,
+	asserta(user:thread_message_hook(_,_,_), Ref), % silence messages
+	catch(read_term(Stream, Term0,
 			[ module(Module)
 			| Options
 			]),
 	      Error,
 	      true),
+	erase(Ref),
 	Restore,
 	pop_operators,
 	(   var(Error)
-	->  !
+	->  Term = Term0
 	;   assert_error(Error, Options),
 	    fail
 	).
@@ -326,10 +299,15 @@ read_source_term_at_location(_, _, Options) :-
 
 assert_error(Error, Options) :-
 	option(error(_), Options), !,
-	(   Error = error(syntax_error(Id), stream(_S, _Line, _LinePos, CharNo))
+	(   (   Error = error(syntax_error(Id),
+			      stream(_S1, _Line1, _LinePos1, CharNo))
+	    ;	Error = error(syntax_error(Id),
+			      file(_S2, _Line2, _LinePos2, CharNo))
+	    )
 	->  message_to_string(error(syntax_error(Id), _), Msg),
 	    assertz(last_syntax_error(CharNo, Msg))
-	;   throw(Error)
+	;   debug(read, 'Error: ~q', [Error]),
+	    throw(Error)
 	).
 assert_error(_, _).
 
