@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -46,7 +46,20 @@
 
 :- multifile
 	http_client:http_convert_data/4,
-	http_client:post_data_hook/3.
+	http_client:post_data_hook/3,
+	json_type/1.
+
+:- predicate_options(http_read_json/3, 3,
+		     [ content_type(any),
+		       false(ground),
+		       null(ground),
+		       true(ground),
+		       value_string_as(oneof([atom, string]))
+		     ]).
+:- predicate_options(reply_json/2, 2,
+		     [ content_type(any),
+		       pass_to(json:json_write/3, 3)
+		     ]).
 
 
 /** <module> HTTP JSON Plugin module
@@ -86,7 +99,7 @@ terms.
 
 http_client:http_convert_data(In, Fields, Data, Options) :-
 	memberchk(content_type(Type), Fields),
-	json_type(Type), !,
+	is_json_type(Type), !,
 	(   memberchk(content_length(Bytes), Fields)
 	->  stream_range_open(In, Range, [size(Bytes)]),
 	    set_stream(Range, encoding(utf8)),
@@ -96,9 +109,35 @@ http_client:http_convert_data(In, Fields, Data, Options) :-
 	).
 
 
+is_json_type(Type) :-
+	json_type(Type), !.
+is_json_type(ContentType) :-
+	json_type(Type),
+	sub_atom(ContentType, 0, _, _, Type), !,
+	strip_utf8(ContentType, Plain),
+	json_type(Plain).
+
+
+%%	strip_utf8(+ContentTypeIn, -ContentType) is det.
+%
+%	Strip an optional  =|;  charset=UTF-8|=.   JSON  data  is always
+%	UTF-8, but some clients seem to insist in sending this.
+
+strip_utf8(ContentType, Plain) :-
+	sub_atom(ContentType, B, _, A, ;),
+	sub_atom(ContentType, _, A, 0, Ext),
+	normalize_space(atom(Charset), Ext),
+	downcase_atom(Charset, 'charset=utf-8'), !,
+	sub_atom(ContentType, 0, B, _, CT),
+	normalize_space(atom(Plain), CT).
+strip_utf8(ContentType, ContentType).
+
+
 %%	json_type(?MIMEType:atom) is semidet.
 %
-%	True if MIMEType is a JSON mimetype.
+%	True if MIMEType is a JSON  mimetype. http_json:json_type/1 is a
+%	multifile  predicate  and  may   be    extended   to  facilitate
+%	non-conforming clients.
 
 json_type('application/jsonrequest').
 json_type('application/json').
@@ -156,7 +195,7 @@ request_to_json(Request, JSON, Options) :-
 	->  true
 	;   domain_error(method, Method)
 	),
-	(   json_type(Type)
+	(   is_json_type(Type)
 	->  true
 	;   domain_error(mimetype, Type)
 	),
