@@ -1,6 +1,4 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@cs.vu.nl
@@ -58,6 +56,10 @@ two additional modules that acts as plugins:
     Loading this library causes http_open/3 to support the =POST= method
     in addition to =GET= and =HEAD=.
 
+    * library(http/http_ssl_plugin)
+    Loading this library causes http_open/3 HTTPS connections.  Relevant
+    options for SLL certificate handling are handled to ssl_context/3.
+
 Here is a simple example to fetch a web-page:
 
   ==
@@ -220,6 +222,8 @@ user_agent('SWI-Prolog <http://www.swi-prolog.org>').
 %		==
 %
 %	@error existence_error(url, Id)
+%	@see ssl_context/3 for SSL related options if
+%	library(http/http_ssl_plugin) is loaded.
 
 http_open(URL, Stream, QOptions) :-
 	meta_options(is_meta, QOptions, Options),
@@ -299,9 +303,9 @@ guarded_send_rec_header(Out, In, Stream, Host, RequestURI, Parts, Options) :-
 	method(Options, MNAME),
 	http_version(Version),
 	format(Out,
-	       '~w ~w HTTP/~w\r\n\
-	       Host: ~w\r\n\
-	       User-Agent: ~w\r\n\
+	       '~w ~w HTTP/~w\r\n\c
+	       Host: ~w\r\n\c
+	       User-Agent: ~w\r\n\c
 	       Connection: close\r\n',
 	       [MNAME, RequestURI, Version, Host, Agent]),
 	x_headers(Options, Out),
@@ -590,7 +594,12 @@ read_header(In, Code, Comment, Lines) :-
 	phrase(first_line(Code, Comment), Line),
 	debug(http(open), '~w ~w', [Code, Comment]),
 	read_line_to_codes(In, Line2),
-	rest_header(Line2, In, Lines), !.
+	rest_header(Line2, In, Lines), !,
+	(   debugging(http(open))
+	->  forall(member(HL, Lines),
+		   debug(http(open), '~s', [HL]))
+	;   true
+	).
 read_header(_, 500, 'Invalid reply header', []).
 
 rest_header("", _, []) :- !.		% blank line: end of header
