@@ -1,11 +1,9 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2010, University of Amsterdam
+    Copyright (C): 1985-2012, University of Amsterdam
 			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
@@ -81,6 +79,7 @@
 	    rdf_atom_md5/3,		% +Text, +Times, -MD5
 
 	    rdf_graph_property/2,	% ?Graph, ?Property
+	    rdf_set_graph/2,		% +Graph, +Property
 	    rdf_graph/1,		% ?Graph
 	    rdf_source/1,		% ?File
 	    rdf_source/2,		% ?DB, ?SourceURL
@@ -102,6 +101,9 @@
 	    rdf_current_ns/2,		% ?Alias, ?URI
 	    rdf_register_ns/2,		% +Alias, +URI
 	    rdf_register_ns/3,		% +Alias, +URI, +Options
+	    rdf_current_prefix/2,	% ?Alias, ?URI
+	    rdf_register_prefix/2,	% +Alias, +URI
+	    rdf_register_prefix/3,	% +Alias, +URI, +Options
 	    rdf_global_id/2,		% ?NS:Name, ?Global
 	    rdf_global_object/2,	% ?Object, ?NSExpandedObject
 	    rdf_global_term/2,		% Term, WithExpandedNS
@@ -176,7 +178,9 @@
 		       convert_typed_literal(callable),
 		       encoding(encoding),
 		       document_language(atom),
-		       namespaces(list(atom))
+		       namespaces(list(atom)),
+		       xml_attributes(boolean),
+		       inline(boolean)
 		     ]).
 :- predicate_options(rdf_save_header/2, 2,
 		     [ graph(atom),
@@ -194,10 +198,7 @@
 	ns/2,
 	rdf_meta_specification/3.	% UnboundHead, Module, Head
 :- dynamic
-	ns/2,			% ID, URL
-	rdf_source/5.		% DB, SourceURL, ModTimeAtLoad, Triples, MD5
-:- volatile
-	rdf_source/5.
+	ns/2.			% ID, URL
 :- discontiguous
 	term_expansion/2.
 
@@ -210,6 +211,16 @@
 		 *	     NAMESPACES		*
 		 *******************************/
 
+%%	rdf_current_prefix(?Alias, ?URI) is nondet.
+%%	rdf_register_prefix(?Alias, ?URI) is det.
+%%	rdf_register_prefix(?Alias, ?URI, +Options) is det.
+%
+%	Forward compatibility. Version 3 deprecates the _ns versions.
+
+rdf_current_prefix(Alias, URI)           :- rdf_current_ns(Alias, URI).
+rdf_register_prefix(Alias, URI, Options) :- rdf_register_ns(Alias, URI, Options).
+rdf_register_prefix(Alias, URI)          :- rdf_register_ns(Alias, URI).
+
 %%	rdf_current_ns(?Alias, ?URI) is nondet.
 %
 %	Query  predefined  namespaces  and    namespaces   defined  with
@@ -257,13 +268,11 @@ rdf_register_ns(Alias, URI) :-
 rdf_register_ns(Alias, URI, _) :-
 	ns(Alias, URI), !.
 rdf_register_ns(Alias, URI, Options) :-
-	ns(Alias, _),
-	(   option(force(Force), Options, false),
-	    Force == true
+	ns(Alias, _), !,
+	(   option(force(true), Options, false)
 	->  retractall(ns(Alias, _)),
 	    assert(ns(Alias, URI))
-	;   option(keep(Keep), Options, false),
-	    Keep == true
+	;   option(keep(true), Options, false)
 	->  true
 	;   throw(error(permission_error(register, namespace, Alias),
 			context(_, 'Already defined')))
@@ -645,7 +654,9 @@ rdf_source_location(Subject, Source) :-
 %	@param KeyValue	Term of the form Key(Value).
 
 rdf_statistics(sources(Count)) :-
-	predicate_property(rdf_source(_,_,_,_,_), number_of_clauses(Count)).
+	rdf_statistics_(graphs(Count)).
+rdf_statistics(graphs(Count)) :-		% version 3 compatibility
+	rdf_statistics_(graphs(Count)).
 rdf_statistics(subjects(Count)) :-
 	rdf_statistics_(subjects(Count)).
 rdf_statistics(properties(Count)) :-
@@ -675,6 +686,8 @@ rdf_statistics(triples_by_file(File, Count)) :-
 	    rdf_statistics_(triples(File, Count))
 	;   rdf_statistics_(triples(File, Count))
 	).
+rdf_statistics(triples_by_graph(Graph, Count)) :- % version 3 compatibility
+	rdf_statistics(triples_by_file(Graph, Count)).
 rdf_statistics(duplicates(Count)) :-
 	rdf_statistics_(duplicates(Count)).
 
@@ -867,24 +880,11 @@ check_loaded_cache(DB, Graphs, _) :-
 
 %%	rdf_load_db(+File) is det.
 %
-%	Load triples from a file created using rdf_save_db/2 and update
-%	the file administration.
+%	Load triples from a file created using rdf_save_db/2.
 
 rdf_load_db(File) :-
 	uri_file_name(URL, File),
-	rdf_load_db_no_admin(File, URL, Graphs),
-	(   (   is_list(Graphs)
-	    ->	member(DB, Graphs)
-	    ;	DB = Graphs
-	    ),
-	    rdf_md5(DB, MD5),
-	    rdf_statistics_(triples(DB, Triples)),
-	    rdf_graph_source_(DB, SourceURL, Modified),
-	    retractall(rdf_source(DB, _, _, _, _)),
-	    assert(rdf_source(DB, SourceURL, Modified, Triples, MD5)),
-	    fail
-	;   true
-	).
+	rdf_load_db_no_admin(File, URL, _Graphs).
 
 
 		 /*******************************
@@ -959,7 +959,6 @@ rdf_load(Spec, M:Options) :-
 	->  do_unload(Graph),
 	    catch(rdf_load_db_no_admin(CacheFile, cache(Graph), Graphs), _, fail),
 	    check_loaded_cache(Graph, Graphs, Modified),
-	    rdf_statistics_(triples(Graph, Triples)),
 	    Action = load
 	;   option(base_uri(BaseURI), Options, Graph),
 	    (	var(BaseURI)
@@ -979,11 +978,9 @@ rdf_load(Spec, M:Options) :-
 			 Cleanup),
 	    save_cache(Graph, SourceURL, Options),
 	    register_file_ns(NSList),
-	    rdf_statistics_(triples(Graph, Triples)),
-	    rdf_md5(Graph, MD5),
-	    assert(rdf_source(Graph, SourceURL, Modified, Triples, MD5)),
 	    format_action(Format, Action)
 	),
+	rdf_statistics_(triples(Graph, Triples)),
 	report_loaded(Action, SourceURL, Graph, Triples, T0, Options).
 
 format_action(triples, load) :- !.
@@ -1309,7 +1306,6 @@ rdf_unload(_).
 do_unload(DB) :-
 	rdf_transaction(rdf_retractall(_,_,_,DB),
 			unload(DB)),
-	retractall(rdf_source(DB, _, _, _, _)),
 	rdf_unset_graph_source(DB).
 
 
@@ -1371,6 +1367,9 @@ modified_graph(SourceURL, Graph) :-
 %
 %	    * hash(Hash)
 %	    Hash is the (MD5-)hash for the content of Graph.
+%	    * modified(Boolean)
+%	    True if the graph is modified since it was loaded or
+%	    rdf_set_graph/2 was called with modified(false).
 %	    * source(Source)
 %	    The graph is loaded from the Source (a URL)
 %	    * source_last_modified(?Time)
@@ -1385,6 +1384,8 @@ rdf_graph_property(Graph, Property) :-
 
 rdf_graph_property_(hash(Hash), Graph) :-
 	rdf_md5(Graph, Hash).
+rdf_graph_property_(modified(Boolean), Graph) :-
+	rdf_graph_modified_(Graph, Boolean, _).
 rdf_graph_property_(source(URL), Graph) :-
 	rdf_graph_source_(Graph, URL, _).
 rdf_graph_property_(source_last_modified(Time), Graph) :-
@@ -1393,6 +1394,17 @@ rdf_graph_property_(source_last_modified(Time), Graph) :-
 rdf_graph_property_(triples(Count), Graph) :-
 	rdf_statistics_(triples(Graph, Count)).
 
+%%	rdf_set_graph(+Graph, +Property) is det.
+%
+%	Set properties of Graph.  Defined properties are:
+%
+%	    * modified(false)
+%	    Set the modified state of Graph to false.
+
+rdf_set_graph(Graph, modified(Modified)) :-
+	must_be(Modified, oneof([false])),
+	rdf_graph_clear_modified_(Graph).
+
 
 %%	save_cache(+DB, +Cache) is det.
 %
@@ -1427,7 +1439,6 @@ assert_triples([H|_], _) :-
 %	statistics.
 
 rdf_reset_db :-
-	retractall(rdf_source(_,_,_,_,_)),
 	rdf_transaction(rdf_reset_db_, reset).
 
 
@@ -1481,7 +1492,8 @@ rdf_reset_db :-
 %			in a term stream(Out).
 
 :- thread_local
-	named_anon/2.			% +Resource, -Id
+	named_anon/2,			% +Resource, -Id
+	inlined/1.			% +Resource
 
 rdf_save(File) :-
 	rdf_save2(File, []).
@@ -1541,6 +1553,7 @@ cleanup_save(Reason,
 	flag(rdf_db_saved_subjects, SavedSubjects, OSavedSubjects),
 	flag(rdf_db_saved_triples, SavedTriples, OSavedTriples),
 	retractall(named_anon(_, _)),
+	retractall(inlined(_)),
 	(   Reason == exit
 	->  print_message(informational,
 			  rdf(saved(File, SavedSubjects, SavedTriples)))
@@ -1861,6 +1874,7 @@ xml_code(0'-).				% Match 0'-
 
 rdf_save_footer(Out) :-
 	retractall(named_anon(_, _)),
+	retractall(inlined(_)),
 	format(Out, '</rdf:RDF>~n', []).
 
 %%	rdf_save_non_anon_subject(+Out, +Subject, +Options)
@@ -1904,7 +1918,12 @@ rdf_save_subject(Out, Subject, DB) :-
 %
 %	@param Indent	Current indentation
 
+rdf_save_subject(_, Subject, _, _, _) :-
+	inlined(Subject), !.
 rdf_save_subject(Out, Subject, BaseURI, Indent, Options) :-
+	do_save_subject(Out, Subject, BaseURI, Indent, Options).
+
+do_save_subject(Out, Subject, BaseURI, Indent, Options) :-
 	graph(Options, DB),
 	findall(Pred=Object, rdf_db(Subject, Pred, Object, DB), Atts0),
 	sort(Atts0, Atts),		% remove duplicates
@@ -1973,7 +1992,7 @@ save_about(Out, BaseURI, Subject) :-
 %	begin tag has already been filled.
 
 save_attributes(Atts, BaseURI, Out, Element, Indent, Options) :-
-	split_attributes(Atts, InTag, InBody),
+	split_attributes(Atts, InTag, InBody, Options),
 	SubIndent is Indent + 2,
 	save_attributes2(InTag, BaseURI, tag, Out, SubIndent, Options),
 	(   InBody == []
@@ -1985,13 +2004,15 @@ save_attributes(Atts, BaseURI, Out, Element, Indent, Options) :-
 	    format(Out, '>~n', [])
 	).
 
-%%	split_attributes(+Attributes, -HeadAttrs, -BodyAttr)
+%%	split_attributes(+Attributes, -HeadAttrs, -BodyAttr, Options)
 %
 %	Split attribute (Name=Value) list into attributes for the head
 %	and body. Attributes can only be in the head if they are literal
 %	and appear only one time in the attribute list.
 
-split_attributes(Atts, HeadAttr, BodyAttr) :-
+split_attributes(Atts, [], Atts, Options) :-
+	option(xml_attributes(false), Options), !.
+split_attributes(Atts, HeadAttr, BodyAttr, _) :-
 	duplicate_attributes(Atts, Dupls, Singles),
 	simple_literal_attributes(Singles, HeadAttr, Rest),
 	append(Dupls, Rest, BodyAttr).
@@ -2074,7 +2095,7 @@ save_attribute(body, Name=Value, BaseURI, Out, Indent, Options) :-
 		(S1 \== S2 ; Name \== P2)
 	    ->  predicate_property(named_anon(_,_), number_of_clauses(N)),
 		atom_concat('bn', N, NodeID),
-		assert(named_anon(Value, NodeID))
+		assertz(named_anon(Value, NodeID))
 	    ;	true
 	    ),
 	    SubIndent is Indent + 2,
@@ -2089,6 +2110,25 @@ save_attribute(body, Name=Value, BaseURI, Out, Indent, Options) :-
 	    rdf_write_id(Out, NameText),
 	    format(Out, '>~n', [])
 	).
+save_attribute(body, Name=Value, BaseURI, Out, Indent, Options) :-
+	option(inline(true), Options),
+	has_attributes(Value, Options),
+	\+ inlined(Value), !,
+	assertz(inlined(Value)),
+	rdf_id(Name, BaseURI, NameText),
+	format(Out, '~N~*|<', [Indent]),
+	rdf_write_id(Out, NameText),
+	SubIndent is Indent + 2,
+	(   rdf_collection(Value)
+	->  save_about(Out, BaseURI, Value),
+	    format(Out, ' rdf:parseType="Collection">~n', []),
+	    rdf_save_list(Out, Value, BaseURI, SubIndent, Options)
+	;   format(Out, '>~n', []),
+	    do_save_subject(Out, Value, BaseURI, SubIndent, Options)
+	),
+	format(Out, '~N~*|</', [Indent]),
+	rdf_write_id(Out, NameText),
+	format(Out, '>~n', []).
 save_attribute(body, Name=Value, BaseURI, Out, Indent, _DB) :-
 	stream_property(Out, encoding(Encoding)),
 	rdf_value(Value, BaseURI, QVal, Encoding),
@@ -2097,6 +2137,10 @@ save_attribute(body, Name=Value, BaseURI, Out, Indent, _DB) :-
 	rdf_write_id(Out, NameText),
 	format(Out, ' rdf:resource="~w"/>', [QVal]).
 
+has_attributes(URI, Options) :-
+	graph(Options, DB),
+	rdf_db(URI, _, _, DB), !.
+
 %%	save_body_literal(+Literal, +NameText, +BaseURI,
 %%			  +Out, +Indent, +Options).
 
@@ -2353,9 +2397,9 @@ source(SourceURL) -->
 	{ uri_file_name(SourceURL, File), !,
 	  file_base_name(File, Base)	% TBD: relative file?
 	},
-	[ ' "~p"'-[Base] ].
+	[ ' "~w"'-[Base] ].
 source(SourceURL) -->
-	[ ' "~p"'-[SourceURL] ].
+	[ ' "~w"'-[SourceURL] ].
 
 into(_, _) --> [].			% TBD
 
