@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -236,15 +236,30 @@ rdfs_ns_label(Resource, Lang, Label) :-
 %%	take_label(+Resource, ?Lang, -Label) is multi.
 %
 %	Get the label to use for a  resource in the give Language. First
-%	tries label_of/3 and if all  fails,   unifies  Label to the last
-%	part of Resource using rdf_split_url/3.
+%	tries label_of/3.  If this fails, break the Resource over # or /
+%	and if all fails, unify Label with Resource.
 
 take_label(Resource, Lang, Label) :-
 	(   label_of(Resource, Lang, Label)
 	*-> true
-	;   rdf_split_url(_, Label, Resource)
+	;   after_char(Resource, '#', Local)
+	->  Label = Local
+	;   after_char(Resource, '/', Local)
+	->  Label = Local
+	;   Label = Resource
 	).
 
+after_char(Atom, Char, Rest) :-
+	State = last(-),
+	(   sub_atom(Atom, _, _, L, Char),
+	    nb_setarg(1, State, L),
+	    fail
+	;   arg(1, State, L),
+	    L \== (-)
+	),
+	sub_atom(Atom, _, L, 0, Rest).
+
+
 %%	label_of(+Resource, ?Lang, ?Label) is nondet.
 %
 %	True if rdf_has(Resource, rdfs:label,   literal(Lang, Label)) is
@@ -257,6 +272,9 @@ label_of(Resource, Lang, Label) :-
 	rdf_equal(rdfs:label, LabelP),
 	rdf_has(Resource, LabelP, literal(lang(Lang, Label)), P),
 	P \== LabelP.
+label_of(Resource, Lang, Label) :-
+	var(Lang),
+	rdf_has(Resource, rdfs:label, literal(type(xsd:string, Label))).
 
 %%	rdfs_class_property(+Class, ?Property)
 %
@@ -278,21 +296,30 @@ rdfs_class_property(Class, Property) :-
 %	parseType="Collection" as well as on Bag, Set and Alt.
 
 rdfs_member(Element, Set) :-
-	rdf_has(Set, rdf:first, _),
+	rdf_has(Set, rdf:first, _), !,
 	rdfs_collection_member(Element, Set).
 rdfs_member(Element, Set) :-
-	rdfs_individual_of(Set, rdfs:'Container'), !,
+	container_class(Class),
+	rdfs_individual_of(Set, Class), !,
 	(   nonvar(Element)
 	->  rdf(Set, Predicate, Element),
 	    rdf_member_property(Predicate, _N)
-	;   between(1, infinite, N),
-	    rdf_member_property(Prop, N),
-	    (	rdf(Set, Prop, Member)
-	    ->	Member = Element
-	    ;	!, fail
-	    )
+	;   findall(N-V, rdf_nth(Set, N, V), Pairs),
+	    keysort(Pairs, Sorted),
+	    member(_-Element, Sorted)
 	).
 
+rdf_nth(Set, N, V) :-
+	rdf(Set, P, V),
+	rdf_member_property(P, N).
+
+:- rdf_meta container_class(r).
+
+container_class(rdf:'Bag').
+container_class(rdf:'Seq').
+container_class(rdf:'Alt').
+
+
 rdfs_collection_member(Element, Set) :-
 	rdf_has(Set, rdf:first, Element).
 rdfs_collection_member(Element, Set) :-
@@ -437,7 +464,7 @@ generate_domain(all_values_from(Class), Individual) :-
 	;   rdfs_individual_of(Individual, Class)
 	).
 generate_domain(some_values_from(Class), Individual) :- % Actually this is
-	rdfs_individual_of(Individual, Class). 		% anything
+	rdfs_individual_of(Individual, Class).		% anything
 generate_domain(union_of(Domains), Individual) :-
 	member(Domain, Domains),
 	generate_domain(Domain, Individual).
