@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <h/kernel.h>
@@ -76,7 +76,7 @@ ws_dispatch(Int FD, Any timeout)
   int ofd = dispatch_fd;
   int fd = (isDefault(FD) ? dispatch_fd :
 	    isNil(FD)	  ? -1
-	    		  : valInt(FD));
+			  : valInt(FD));
 
 					/* No context: wait for input */
 					/* timeout */
@@ -93,10 +93,12 @@ ws_dispatch(Int FD, Any timeout)
     { to.tv_sec = 0;
       to.tv_usec = 250000;
     } else if ( isInteger(timeout) )
-    { to.tv_sec  = valInt(timeout) / 1000;
-      to.tv_usec = valInt(timeout) % 1000;
-    } else /* if ( isReal(timeout) ) */
-    { double v = valReal(timeout);
+    { double v;
+
+      if ( isInteger(timeout) )
+	v = (double)valInt(timeout)/1000.0;
+      else
+	v = valReal(timeout);
 
       to.tv_sec  = (long)v;
       to.tv_usec = (long)(v * 1000000.0) % 1000000;
@@ -138,9 +140,10 @@ ws_dispatch(Int FD, Any timeout)
   DEBUG(NAME_dispatch, Cprintf("Dispatch: timeout = %s, tid = %d\n",
 			       pp(timeout), tid));
 
-  pceMTLock(LOCK_PCE);
-  RedrawDisplayManager(TheDisplayManager());
-  pceMTUnlock(LOCK_PCE);
+  if ( pceMTTryLock(LOCK_PCE) )
+  { RedrawDisplayManager(TheDisplayManager());
+    pceMTUnlock(LOCK_PCE);
+  }
 					/* All callbacks must be locked! */
   XtAppProcessEvent(ThePceXtAppContext,
 		    XtIMXEvent|XtIMTimer|XtIMAlternateInput);
@@ -177,8 +180,10 @@ ws_discard_input(const char *msg)
   { char buf[1024];
 
     Cprintf("%s; discarding input ...", msg);
-    read(dispatch_fd, buf, sizeof(buf));
-    Cprintf("ok\n");
+    if ( read(dispatch_fd, buf, sizeof(buf)) >= 0 )
+      Cprintf("ok\n");
+    else
+      Cprintf("failed\n");
   }
 }
 
