@@ -143,9 +143,7 @@ typedef struct
 
 static stream_context *
 getStreamContext(IOSTREAM *s)
-{ Symbol symb;
-
-  if ( !(symb = lookupHTable(streamContext, s)) )
+{ if ( !s->context )
   { stream_context *ctx = allocHeapOrHalt(sizeof(*ctx));
 
     DEBUG(1, Sdprintf("Created ctx=%p for stream %p\n", ctx, s));
@@ -154,15 +152,20 @@ getStreamContext(IOSTREAM *s)
     ctx->filename = NULL_ATOM;
     ctx->flags = 0;
     addHTable(streamContext, s, ctx);
-
-    return ctx;
+    s->context = ctx;
   }
 
-  return symb->value;
+  return (stream_context*)s->context;
 }
 
+static stream_context *
+getExistingStreamContext(IOSTREAM *s)
+{ return (stream_context*)s->context;
+}
 
-void
+/* MT: Must be called locked */
+
+static void
 aliasStream(IOSTREAM *s, atom_t name)
 { stream_context *ctx;
   Symbol symb;
@@ -197,11 +200,12 @@ unaliasStream(IOSTREAM *s, atom_t name)
 
   if ( name )
   { if ( (symb = lookupHTable(streamAliases, (void *)name)) )
-    { deleteSymbolHTable(streamAliases, symb);
+    { stream_context *ctx;
 
-      if ( (symb=lookupHTable(streamContext, s)) )
-      { stream_context *ctx = symb->value;
-	alias **a;
+      deleteSymbolHTable(streamAliases, symb);
+
+      if ( (ctx=getExistingStreamContext(s)) )
+      { alias **a;
 
 	for(a = &ctx->alias_head; *a; a = &(*a)->next)
 	{ if ( (*a)->name == name )
@@ -220,9 +224,10 @@ unaliasStream(IOSTREAM *s, atom_t name)
       PL_unregister_atom(name);
     }
   } else				/* delete them all */
-  { if ( (symb=lookupHTable(streamContext, s)) )
-    { stream_context *ctx = symb->value;
-      alias *a, *n;
+  { stream_context *ctx;
+
+    if ( (ctx=getExistingStreamContext(s)) )
+    { alias *a, *n;
 
       for(a = ctx->alias_head; a; a=n)
       { Symbol s2;
@@ -1748,9 +1753,20 @@ PRED_IMPL("set_stream", 2, set_stream, 0)
 	if ( !PL_get_atom_ex(a, &type) )
 	  return FALSE;
 	if ( type == ATOM_text )
-	{ s->flags |= SIO_TEXT;
+	{ if ( false(s, SIO_TEXT) && Ssetenc(s, LD->encoding, NULL) != 0 )
+	  { PL_error(NULL, 0, NULL, ERR_PERMISSION,
+		     ATOM_encoding, ATOM_stream, stream);
+	    goto error;
+	  }
+	  s->flags |= SIO_TEXT;
 	} else if ( type == ATOM_binary )
-	{ s->flags &= ~SIO_TEXT;
+	{ if ( true(s, SIO_TEXT) && Ssetenc(s, ENC_OCTET, NULL) != 0 )
+	  { PL_error(NULL, 0, NULL, ERR_PERMISSION,
+		     ATOM_encoding, ATOM_stream, stream);
+	    goto error;
+	  }
+
+	  s->flags &= ~SIO_TEXT;
 	} else
 	{ PL_error("set_stream", 2, NULL, ERR_DOMAIN,
 		   ATOM_type, a);
@@ -2999,6 +3015,8 @@ static const opt_spec open4_options[] =
 };
 
 
+/* MT: openStream() must be called unlocked */
+
 IOSTREAM *
 openStream(term_t file, term_t mode, term_t options)
 { GET_LD
@@ -3161,7 +3179,10 @@ openStream(term_t file, term_t mode, term_t options)
   }
 
   if ( alias != NULL_ATOM )
+  { LOCK();
     aliasStream(s, alias);
+    UNLOCK();
+  }
   if ( !reposition )
     s->position = NULL;
 
@@ -3244,7 +3265,7 @@ pl_see(term_t f)
   if ( !PL_get_atom_ex(f, &a) )
     return FALSE;
 
-  LOCK();
+  PL_LOCK(L_SEETELL);
   if ( get_stream_handle(a, &s, SH_ALIAS|SH_UNLOCKED) )
   { Scurin = s;
     goto ok;
@@ -3261,7 +3282,7 @@ pl_see(term_t f)
   mode = PL_new_term_ref();
   PL_put_atom(mode, ATOM_read);
   if ( !(s = openStream(f, mode, 0)) )
-  { UNLOCK();
+  { PL_UNLOCK(L_SEETELL);
     return FALSE;
   }
 
@@ -3270,7 +3291,7 @@ pl_see(term_t f)
   Scurin = s;
 
 ok:
-  UNLOCK();
+  PL_UNLOCK(L_SEETELL);
 
   return TRUE;
 }
@@ -3327,7 +3348,7 @@ do_tell(term_t f, atom_t m)
   if ( !PL_get_atom_ex(f, &a) )
     return FALSE;
 
-  LOCK();
+  PL_LOCK(L_SEETELL);
   if ( get_stream_handle(a, &s, SH_UNLOCKED) )
   { Scurout = s;
     goto ok;
@@ -3344,7 +3365,7 @@ do_tell(term_t f, atom_t m)
   mode = PL_new_term_ref();
   PL_put_atom(mode, m);
   if ( !(s = openStream(f, mode, 0)) )
-  { UNLOCK();
+  { PL_UNLOCK(L_SEETELL);
     return FALSE;
   }
 
@@ -3353,7 +3374,7 @@ do_tell(term_t f, atom_t m)
   Scurout = s;
 
 ok:
-  UNLOCK();
+  PL_UNLOCK(L_SEETELL);
   return TRUE;
 }
 
